<?php
class Anc_Library_Http_ClientWrapper
{
    /**
    * @var CbcZend_Http_Client
    */
    protected $_client;

    protected $_maxSize;

    //Unused
    protected $_removeHtmlNoise = TRUE;

    protected $_commonRequestHeaders = array(
        'Pragma' => 'nocache',
        'Accept-Charset' => 'utf-8',
    );

    protected $_encodingDetectOrder = array(
        'UTF-8',
        'ISO-8859-1',
        'CP1251',
    );

    public function __construct()
    {
        Anc_Application::autoloadLib('CbcZend_Http_Client');
        $this->_client = new CbcZend_Http_Client;

        $appConfig = Anc_Application::app()->getConfig();
        $config = array(
            'timeout' => isset($appConfig->http_client->timeout)
                ? $appConfig->http_client->timeout : 10,
            'fallback_stream_tmp_dir' => isset($appConfig->http_client->fallback_stream_tmp_dir)
                ? $appConfig->http_client->fallback_stream_tmp_dir
                : ANC_APPLICATION_PATH . '/tmp',
            'curloptions' => array(
                CURLOPT_SSL_VERIFYPEER => FALSE,
                CURLOPT_SSL_VERIFYHOST => FALSE,
            ),
        );
        if (isset($appConfig->http_client->stream_tmp_dir)) {
            $config['stream_tmp_dir'] = $appConfig->http_client->stream_tmp_dir;
        }

        // Set cURL adapter if cURL extension supported:
        if (extension_loaded('curl')) {
            Anc_Application::autoloadLib('CbcZend_Http_Client_Adapter_Curl');
            $this->_client->setAdapter(new CbcZend_Http_Client_Adapter_Curl);
        }

        $this->_client->setConfig($config);
    }

    public function get($url, $getParams=null)
    {
        if ($getParams) {
            $this->_client->setParameterGet($getParams);
        }
        return $this->request($url, 'GET');
    }

    public function post($url, $postData=null, $getParams=null)
    {
        if ($postData) {
            $this->_client->setParameterPost($postData);
        }
        if ($getParams) {
            $this->_client->setParameterGet($getParams);
        }
        return $this->request($url, 'POST');
    }

    public function request($url, $method=null)
    {
        $this->_client->setStream();

        $this->_client->setUri($url);

        //Add common headers
        $this->_client->setHeaders($this->_commonRequestHeaders);

        try {
            /**
            * @var CbcZend_Http_Response
            */
            $response = $this->_client->request($method);
        } catch (CbcZend_Http_Client_Exception $e) {
            Anc_Application::autoload('Anc_Library_Http_Exception');
            throw new Anc_Library_Http_Exception($e->getMessage());
        } catch (CbcZend_Uri_Exception $e) {
            Anc_Application::autoload('Anc_Library_Http_Exception');
            throw new Anc_Library_Http_Exception($e->getMessage());
        }
        if ($response->isError()) {
            Anc_Application::autoload('Anc_Library_Http_Exception');
            throw new Anc_Library_Http_Exception($response->getMessage());
        }

        if (isset($this->_maxSize)
            && $response instanceof CbcZend_Http_Response_Stream
        ) {
            $size = @filesize($response->getStreamName());
            if (FALSE !== $size && $size > $this->_maxSize) {
                Anc_Application::autoload('Anc_Library_Http_Exception');
                throw new Anc_Library_Http_Exception('Maximum size exceeeded');
            }
        }

        $result = $this->filterResponse($response);

        return $result;
    }

    public function filterResponse(CbcZend_Http_Response $response)
    {
        $result = $response->getBody();
        $encoding = NULL;

        if (preg_match('~<meta\b.*?\bhttp-equiv="Content-Type".*?\bcontent="(.*?)"~is', $result, $matches)) {
            $encoding = $this->_extractEncoding($matches[1]);
        }

        if ('' == $encoding) {
            $contentType = strtolower($response->getHeader('Content-Type'));
            if ('' != $contentType && strpos($contentType, 'text/html')===0) {
                $encoding = $this->_extractEncoding($contentType);
            }
        }

        if ('' == $encoding) {
            $encoding = function_exists('mb_detect_encoding')
                ? strtolower(mb_detect_encoding($result, $this->_encodingDetectOrder))
                : null;
        }

        if (!in_array($encoding, array('utf-8', 'utf8'))) {
            if ('' != $encoding) {
                $result = iconv($encoding, 'utf-8', $result);
            }

            if ('' == trim($result)) {
                $result = $response->getBody();
            }
        }

        return wp_check_invalid_utf8($result, TRUE);
    }

    protected function _extractEncoding($contentType)
    {
        if (preg_match('/\bcharset=(.*)(;|$)/is', $contentType, $matches)) {
            $result = strtolower(trim($matches[1]));
        }
        else {
            $result = '';
        }

        return $result;
    }

    //
    // Getters & setters
    //

    public function getClient()
    {
        return $this->_client;
    }

    public function getRemoveHtmlNoise()
    {
        return $this->_removeHtmlNoise;
    }

    public function setRemoveHtmlNoise($value)
    {
        $this->_removeHtmlNoise = $value;
    }

    public function getMaxSize()
    {
        return $this->_maxSize;
    }

    public function setMaxSize($value)
    {
        $this->_maxSize = $value;
    }

    public function getUrl($noQuery=FALSE)
    {
        // Reset default HTTP ports
        /**
        * @var CbcZend_Uri_Http
        */
        $uri = clone $this->_client->getUri();

        if (('http' == $uri->getScheme() && 80 == $uri->getPort())
            || ('https' == $uri->getScheme() && 443 == $uri->getPort())
        ) {
            $uri->setPort('');
        }
        if ($noQuery) {
            $uri->setQuery('');
        }

        $result = $uri->getUri();

        return $result;
    }

    public function checkCookie($name)
    {
        foreach ($this->_client->getCookieJar() as $cookie) {
            if (FALSE !== stripos($cookie->getName(), $name)) {
                return $cookie->getValue();
            }
        }
    }
}