<?php

/*
Plugin Name: CB Chief - Developer
Plugin URI: http://cbchief.com/
Description: CB Chief Builds Clickbank Affiliate Niche Blogs at The Push of a Button...
Author: CB Chief
Developer: CB Chief
Version: 1.0.0
Author URI: http://cbchief.com/

*/
require_once("validate_cbchief_developer.php");
validatePlugin_cbchief_developer();

require_once dirname(__FILE__) . '/library/compatibility.php';

//Register plug-in:
Cbc_WpPlugin::register();

class Cbc_WpPlugin
{
//------------------------------------------------------------------------------
// Static members
//------------------------------------------------------------------------------
    const PLUGIN_BASE_URL = 'http://cbchief.com/';
    const AUTHOR_BASE_URL = 'http://cbchief.com/';

    /**
    * Version tag. Used for CSS & JS versioning.
    */
    const PLUGIN_VERSION = '1.0.0';

    /**
    * Unique slug
    */
    const PLUGIN_SLUG = 'CBChief';

    /**
    * Product name for auto-update system
    */
    const PRODUCT_NAME = 'cb_chief';

    /**
    * User-friendly title
    */
    const PLUGIN_TITLE = 'CB Chief - Developer';

    /**
    * Menu item text
    */
    const PLUGIN_MENU_LABEL = 'CB Chief - Developer';

    /**
    * Unique prefix
    */
    const PREFIX = 'cbc';

    /**
    * Required capability to access plug-in pages
    */
    const PLUGIN_CAPABILITY = 'publish_posts';

    const DEFAULT_CLOAK_DIR = 'visit';

    static protected $_instance;

    static protected $_postTemplateVars;

    static protected $_postMetaFields = array(
        'merchant_id'           => TRUE,
        'merchant_url'          => TRUE,
        'thumbnail_timestamp'   => FALSE,
    );

    /**
    * Virtual fields to serve via get_post_metadata filter
    */
    static protected $_virtualPostMetaFields = array(
        'thumbnail',
        'thumbnail_url',
        'affiliate_url',
    );

    static protected $_redirectSlug;

    static protected $_updater;
    static protected $_updaterCronHook;
    const UPDATE_CHECK_UNTERVAL = 12;

    /**
    * Registers plug-in module
    */
    static public function register()
    {
        // Auto update
        require dirname(__FILE__) . '/plugin-updates/plugin-update-checker.php';
        self::$_updater = new PluginUpdateChecker_1_3(
            'http://wppluginupdate.com/check/' . self::PRODUCT_NAME . '.json',
            __FILE__,
            self::PRODUCT_NAME,
            0
        );
        self::$_updaterCronHook = 'auto_check_updates-' . self::PRODUCT_NAME;
        add_action(self::$_updaterCronHook, array(self::$_updater, 'checkForUpdates'));

        $class = get_class();

        register_activation_hook(__FILE__, array($class, 'activate'));
        register_deactivation_hook(__FILE__, array($class, 'deactivate'));

        // Admin actions
        if (is_admin()) {
            add_action('admin_init', array($class, 'adminInit'));
            add_action('admin_init', array($class, 'autoCheckUpdates'));
            add_action('admin_menu', array($class, 'adminMenu'));
            add_action('wp_ajax_cbc', array($class, 'adminAjax'));
        } else {
            // Frontend actions
            add_action('wp_print_styles',  array($class, 'wpPrintStyles'));
            add_action('wp_enqueue_scripts',  array($class, 'wpEnqueueScripts'));
            // Post filter actions
            add_action('loop_start', array($class, 'loopStart'));

            // URL listener
            if (self::checkUrl()) {
                add_filter('wp_loaded', array($class, 'filterUrl'));
            }
        }

        // Cron action
        add_action('cbc_prepare_thumbs', array($class, 'prepareThumbs'));

        // Filter for calling post template from plug-ins
        add_action('cbc_exec_post_template', array($class, 'execPostTemplate'));

        // Post metadata filter
        add_filter('get_post_metadata', array($class, 'getPostMetadata'), 10, 4);

        self::addLibraryIncludePath();
    }

    /**
    * Plug-in activate hook
    */
    static public function activate($networkWide)
    {
        if (version_compare(PHP_VERSION, '5.0.0', '<')) {
            // Display Error Message
            self::throwInstallError(sprintf('Sorry, but %s requires PHP 5.0 or newer. Your version is %s. Please, ask your web host to upgrade to PHP 5.0.',
                self::PLUGIN_TITLE, phpversion()));
        }

        // Requires Wordpress 3.0+
        global $wp_version;
        if (version_compare($wp_version, '3.0', '<' )) {
            self::throwInstallError(sprintf('Sorry, but %s requires Wordpress 3.0 or newer. Your version is %s. Please, upgrade to the latest version of Wordpress.',
                self::PLUGIN_TITLE, $wp_version));
        }

        // Require mcrypt extension
        if (!extension_loaded('mcrypt') || !function_exists('mcrypt_decrypt')) {
            self::throwInstallError(sprintf('Sorry, but %s requires mcrypt PHP extension. Please ask your web host to enable mcrypt extension.',
                self::PLUGIN_TITLE));
        }

        // Check session support
        if (!session_id()) {
            $oldErrorReporting = error_reporting();
            $oldDisplayErrors = ini_get('display_errors');
            $oldHtmlErrors = ini_get('html_errors');

            error_reporting(E_ALL);
            ini_set('display_errors', '1');
            ini_set('html_errors', '0');

            ob_start();
            $result = session_start();
            $error = ob_get_clean();

            error_reporting($oldErrorReporting);
            ini_set('display_errors', $oldDisplayErrors);
            ini_set('html_errors', $oldHtmlErrors);

            if (!$result || !session_id() || '' != $error) {
                // Deactivate Plugin
                if(function_exists('deactivate_plugins')) {
                    deactivate_plugins(plugin_basename(__FILE__), true);
                }

                $msg = sprintf('Sorry but %s requires PHP sessions. There was an error starting a session. Please, contact your hosting support in order to resolve the problem.',
                     self::PLUGIN_TITLE);
                if ('' != $error) {
                    $msg .= ' The error message logged: ' . $error;
                }

                self::throwInstallError($msg);
            }
        }

        wp_schedule_event(time(), 'daily', 'cbc_prepare_thumbs');
    }

    static public function throwInstallError($message)
    {
        if(function_exists('deactivate_plugins')) {
            deactivate_plugins(plugin_basename(__FILE__), true);
        }

        trigger_error($message, E_USER_ERROR);
    }

    /**
    * Plug-in deactivation hook
    */
    static public function deactivate()
    {
        wp_clear_scheduled_hook('cbc_prepare_thumbs');
    }

    static public function addLibraryIncludePath()
    {
        $includePaths = explode(PATH_SEPARATOR, get_include_path());
        $path = realpath(dirname(__FILE__) . '/library');
        if (!in_array($path, $includePaths)) {
            foreach ($includePaths as $key=>$item) {
                if ('.' == $item) {
                    unset($includePaths[$key]);
                    break;
                }
            }
            array_unshift($includePaths, $path);
            array_unshift($includePaths, '.');
            set_include_path(implode(PATH_SEPARATOR, $includePaths));
        }
    }

    static public function loadApplication()
    {
        self::addLibraryIncludePath();
        require_once(self::getInstance()->getPluginPath()
            . '/wp-module/Application.php');
    }

    static public function checkUrl()
    {
        $path = explode('/', substr($_SERVER['REQUEST_URI'], strlen(self::getHomePath())));

        if ('index.php' == $path[0]) {
            array_shift($path);
        }

        if (2 != count($path)
            || self::getCloakDir() !== $path[0]
            || '' == $path[1]
        ) {
            return FALSE;
        }

        self::$_redirectSlug = $path[1];

        return TRUE;
    }

    static public function filterUrl()
    {
        self::loadApplication();

        $service = Cbc_WpModule_Application::getModel('Linker')
            ->redirect(self::$_redirectSlug);
    }

    static public function getCloakDir()
    {
        return get_option(self::PREFIX . '_cloak_dir', self::DEFAULT_CLOAK_DIR);
    }

    static public function getHomePath()
    {
         return trailingslashit((string) parse_url(site_url(), PHP_URL_PATH));
    }

    static public function adminInit()
    {
        if (!session_id()) {
            // Workaround for weird problem (session_id not read from cookies)
            if (isset($_COOKIE[session_name()])) {
                session_id($_COOKIE[session_name()]);
            }
            session_start();
        }
    }

    static public function autoCheckUpdates()
    {
        $updater = self::$_updater;
        $state = $updater->getUpdateState();

        $shouldCheck = (empty($state)
            || !isset($state->lastCheck)
            || ((time() - $state->lastCheck) >= (self::UPDATE_CHECK_UNTERVAL * 3600)))
            && !wp_next_scheduled(self::$_updaterCronHook);

        if ($shouldCheck) {
            wp_schedule_single_event(time(), self::$_updaterCronHook);
            spawn_cron();
        }
    }

    /**
    * admin_menu hook
    */
    static public function adminMenu()
    {
        self::getInstance()->_registerAdminPages();
    }

    static public function adminAjax()
    {
        self::getInstance()->dispatchAjax();
    }

    /**
    * wp_print_styles hook
    */
    static public function wpPrintStyles()
    {
        wp_enqueue_style('clickbank_connector/post.css',
            self::getInstance()->getPluginBaseUrl() . '/template/post.css',
            array(),
            date('YmdHis', filemtime(dirname(__FILE__) . '/template/post.css')));
    }

    /**
    * wp_enqueue_scripts hook
    */
    static public function wpEnqueueScripts()
    {

    }

    static public function loopStart()
    {
        $class = get_class();

        add_filter('the_post', array($class, 'filterPost'));
        add_filter('the_content_more_link', array($class, 'filterContentMoreLink'));
        add_filter('the_content', array($class, 'filterContent'), -300);
    }

    static public function filterPost($post)
    {
        self::$_postTemplateVars = NULL;

        if ('post' != $post->post_type) {
            return;
        }

        $postId = $post->ID;
        $ver = get_post_meta($postId, 'cbc_version', TRUE);
        if ($ver < '03.00.00') {
            return;
        }

        $vars = array();
        foreach (self::$_postMetaFields as $metaName => $required) {
            $val = get_post_meta($postId, self::PREFIX . '_' . $metaName, TRUE);
            if ('' == $val) {
                if ($required) {
                    return;
                } else {
                    $val = NULL;
                }
            }
            $vars[$metaName] = $val;
        }
        $vars['more_link'] = FALSE;
        $vars['post'] = $post;
        $vars['post_id'] = $postId;

        self::$_postTemplateVars = $vars;

        return;
    }

    static public function filterContentMoreLink($value)
    {
        if (self::$_postTemplateVars) {
            self::$_postTemplateVars['moreLink'] = TRUE;
        }

        return $value;
    }

    static public function filterContent($value)
    {
        if (self::$_postTemplateVars) {
            self::$_postTemplateVars['content'] = $value;
            ob_start();
            self::getInstance()->dispatch('template');
            $value = ob_get_clean();
        }

        return $value;
    }

    static public function execPostTemplate($post)
    {
        self::filterPost($post);
        $post->post_content = self::filterContent($post->post_content);
    }

    static public function prepareThumbs()
    {
        self::getInstance()->dispatch('cron', 'prepare-thumbs');
    }

    static public function getPostTemplateVars()
    {
        return self::$_postTemplateVars;
    }

    static public function getPostMetadata($value, $objectId, $metaKey, $single)
    {
        if ('cbc_' != substr($metaKey, 0, 4)) {
            return $value;
        }

        $key = substr($metaKey, 4);
        if (!in_array($key, self::$_virtualPostMetaFields)) {
            return $value;
        }

        $vars = array();
        foreach (self::$_postMetaFields as $metaName => $required) {
            $val = get_post_meta($objectId, self::PREFIX . '_' . $metaName, TRUE);
            if ('' == $val) {
                if ($required) {
                    return $value;
                } else {
                    $val = NULL;
                }
            }
            $vars[$metaName] = $val;
        }
        $vars['post_id'] = $objectId;

        self::loadApplication();
        $result = Cbc_WpModule_Application::getModel('VirtualMeta')
            ->getValue($key, $vars);

        if (!is_null($result)) {
            if ((!$single && !is_array($result))
                || $single && is_array($result)
            ) {
                $result = array($result);
            }
            $value = $result;
        }

        return $value;
    }

    /**
    * Singleton pattern
    *
    * @return Cbc_WpPlugin Singleton instance
    */
    static public function getInstance()
    {
        if (!self::$_instance) {
            self::$_instance = new self;
        }

        return self::$_instance;
    }

//------------------------------------------------------------------------------
// Object members
//------------------------------------------------------------------------------
    protected $_pluginBaseUrl;
    protected $_pluginPath;

    protected function __construct()
    {
        $this->_pluginBaseUrl = trailingslashit(get_bloginfo('wpurl')) . PLUGINDIR
            . '/' . dirname(plugin_basename(__FILE__));

        $this->_pluginPath = dirname(__FILE__);
    }

    /**
    * Register plug-in admin pages and menus
    */
    protected function _registerAdminPages()
    {
       
        add_menu_page(self::PLUGIN_TITLE, self::PLUGIN_MENU_LABEL,
            self::PLUGIN_CAPABILITY, self::PLUGIN_SLUG,
            array($this, 'dispatchAdmin'),
            self::getInstance()->getPluginBaseUrl() . '/images/icon.png'
        );

        add_submenu_page(self::PLUGIN_SLUG, self::PLUGIN_TITLE,
            'Admin', self::PLUGIN_CAPABILITY,
            self::PLUGIN_SLUG,
            array($this, 'dispatchAdmin')
        );
        add_action('admin_head-toplevel_page_' . self::PLUGIN_SLUG,
            array($this, 'dispatch__admin__head'));
        add_submenu_page(self::PLUGIN_SLUG, self::PLUGIN_TITLE,
            'Options', self::PLUGIN_CAPABILITY,
            self::PLUGIN_SLUG . '-options',
            array($this, 'dispatch__admin__options')
        );
        add_action('admin_head-toplevel_page_' . self::PLUGIN_SLUG . '-options',
            array($this, 'dispatch__admin__head'));
    }

    /**
    * Intercepts WP requests
    */
    public function __call($method, $arguments)
    {
        if ('dispatch__' == substr($method, 0, 10)) {
            $parts = explode('__', $method);
            $controller = @$parts[1];
            $action = @$parts[2];

            $controller = $this->_prepareMvcName($controller);
            $action = $this->_prepareMvcName($action);

            $this->dispatch($controller, $action);
        }
        else {
            throw new Exception("Method {$method} not found.");
        }
    }

    protected function _prepareMvcName($name)
    {
        return str_replace('_', '-', $name);
    }

    /**
    * Dispatches user request to WP Module
    *
    * @param string $controller Controller name
    * @param string $action Action name
    */
    public function dispatch($controller='index', $action='index',
        $ajax = FALSE)
    {
        self::addLibraryIncludePath();

        require_once($this->_pluginPath . '/wp-module/Application.php');
        Cbc_WpModule_Application::dispatch($controller, $action, $ajax);
    }

    public function dispatchAdmin()
    {
        $controller = @$_GET['controller'];
        if (empty($controller)) $controller = 'admin';
        $action = @$_GET['action'];
        if (empty($action)) $action = 'index';

        $this->dispatch($controller, $action);
    }

    public function dispatchAjax()
    {
        $controller = @$_POST['controller'];
        if (empty($controller)){
            $controller = 'index';
        }
        $action = @$_POST['ajaxAction'];
        if (empty($action)) {
            $action = 'index';
        }

        $this->dispatch($controller, $action, TRUE);

        //exit();
    }

    public function getPluginBaseUrl()
    {
        return $this->_pluginBaseUrl;
    }

    public function getPluginPath()
    {
        return $this->_pluginPath;
    }
}