<?php
define('CBC_WPMODULE_PATH', dirname(__FILE__));

abstract class Cbc_WpModule_Application
{
    const CONTROLLER_NAMESPACE  = 'Cbc_WpModule_Controller';
    const CONTROLLER_FOLDER     = 'controllers';
    const MODEL_NAMESPACE       = 'Cbc_WpModule_Model';
    const MODEL_FOLDER          = 'models';
    const FORM_NAMESPACE        = 'Cbc_WpModule_Form';
    const FORM_FOLDER           = 'forms';

    static protected $_models = array();

    static public function dispatch($controller='index', $action='index',
        $ajax = FALSE)
    {
        if (empty($controller)) {
            $controller = 'index';
        } else {
            $controller = strtolower($controller);
        }
        if (empty($action)) {
            $action = 'index';
        } else {
            $action = strtolower($action);
        }

        $controllerName = self::camelize($controller);

        $controllerClass = $controllerName . ($ajax? 'AjaxController' : 'Controller');
        $controllerObject = self::_factory($controllerClass,
            self::CONTROLLER_NAMESPACE, self::CONTROLLER_FOLDER);

        $controllerObject->preDispatch($action);

        $actionName = self::camelize($action, FALSE);
        $method = $actionName . ('Action');
        if (!$ajax) {
            $controllerObject->setViewScript($controller . '/' . $action . '.phtml');
        }

        $controllerObject->$method();

        $viewScript = $controllerObject->getViewScript();
        if (!empty($viewScript)) {
            /**
            * @var Cbc_WpModule_View_Engine
            */
            $view = $controllerObject->getView();

            if (($layout = $view->layout)
                && ($layoutScript = $controllerObject->getLayoutScript())
            ) {
                $layout->content = $view->render($viewScript);
                $layout->renderDirect($layoutScript);
            } else {
                $view->renderDirect($viewScript);
            }
        }

        $controllerObject->postDispatch($action);

        if ($ajax) {
            exit;
        }
    }

    static public function getModel($model)
    {
        if (!isset(self::$_models[$model])) {
            self::$_models[$model] = self::_factory($model,
                self::MODEL_NAMESPACE, self::MODEL_FOLDER);
        }

        return self::$_models[$model];
    }

    static public function getForm($form)
    {
        return self::_factory($form,
                self::FORM_NAMESPACE, self::FORM_FOLDER);
    }

    /**
    * Creates concrete controller object
    *
    * @param string $class
    * @return Cbc_WpModule_Controller_Front
    */
    static protected function _factory($class, $namespace, $folder)
    {
        require_once realpath(CBC_WPMODULE_PATH . '/' . $folder . '/' . $class . '.php');
        $class = $namespace . '_' . $class;
        return new $class;
    }

    static public function camelize($string, $first=TRUE)
    {
        $string = ucwords(str_replace('-', ' ', $string));
        $string = str_replace(' ', '', $string);
        if (!$first) {
            $string = strtolower($string[0]) . substr($string, 1);
        }

        return $string;
    }

    static public function url($action=NULL, $controller=NULL, $parameters=NULL)
    {
        $get = is_null($parameters)
            ? array()
            : $parameters;
        $get['page'] = Cbc_WpPlugin::PLUGIN_SLUG;
        $get['action'] = $action;
        $get['controller'] = $controller;
        foreach ($get as $name=>$value) {
            if (is_null($value)) {
                unset($get[$name]);
            }
        }

        return dirname($_SERVER['SCRIPT_NAME']) . '/admin.php?' . http_build_query($get, '', '&');
    }
}